
import processing.video.*;

Capture captureImage;
PImage sized;
PImage ready;
PImage result;
PImage flipped;
int W = 160;
int H = 120;
int FACTOR=2;
boolean doReset = true;

PImage alphaImage = new PImage(W,H);

int idx(int x, int y) {
  return (y*W+x)%(W*H);  
}

int idxf(int x, int y) {
  return (y*W*FACTOR+x)%(W*H*FACTOR*FACTOR);  
}


int STEPS=16;
int bucket(color c) {
  int h = (int)(STEPS*hue(c)/256.0);
  int s = (int)(STEPS*saturation(c)/256.0);
  int b = (int)(STEPS*brightness(c)/256.0);
  return h+STEPS*s+STEPS*STEPS*b;
}

int TOPBUCK = STEPS*STEPS*STEPS;

float [] fStore = new float[TOPBUCK];
float [] bStore = new float[TOPBUCK];
float fCount = 0;
float bCount = 0;

void setup() 
{
  noLoop();
  size(W*FACTOR,H*FACTOR);
  captureImage = new Capture(this, W*FACTOR,H*FACTOR);
  sized = new PImage(W,H);
  ready = new PImage(W*FACTOR,H*FACTOR);
  result = new PImage(W*FACTOR,H*FACTOR);
  flipped = new PImage(W*FACTOR,H*FACTOR);
  noStroke();
  smooth();
}

void keyPressed() {
  doReset = true;
}

void select(PImage target) {
  for(int x=0; x<W; x++) {
    for(int y=0; y<H; y++) {
      float d = 1.0-dist(x,y,H/2,W/2)/W;
      target.pixels[idx(x,y)] = color(512*pow(d,8));

    } 
  }
  target.updatePixels(); 
}

void analyze(PImage original, PImage msk)  {
  fCount = 0;
  bCount = 0;
  for(int i=0; i<fStore.length; i++) {
    fStore[i] = 1.0;
    bStore[i] = 1.0;
  }
  for(int x=0; x<W; x++) {
    for(int y=0; y<H; y++) {
      color src = original.pixels[idx(x,y)];
      color a = msk.pixels[idx(x,y)];
      int buck = bucket(src);
      float fPoints = red(a)/256.0;
      fStore[buck] += fPoints;
      bStore[buck] += 1.0-fPoints;
      fCount += fPoints;
      bCount += 1-fPoints;
    } 
  }

  if(fCount < W*H/8) {
    doReset = true; 
  }
}

void reselect(PImage original, PImage msk) {
  float bal = bCount / fCount;
  for(int x=0; x<W; x++) {
    for(int y=0; y<H; y++) {
      color src = original.pixels[idx(x,y)];
      int buck = bucket(src);
      float amount = (bal*fStore[buck])/(bal*fStore[buck]+bStore[buck]);
      amount *= amount;
      msk.pixels[idx(x,y)] = color(256*amount);
    } 
  }
  msk.updatePixels();
}

void resynth(PImage original, PImage bigMsk,PImage into) {
  for(int x=0; x<W*FACTOR; x++) {
    for(int y=0; y<H*FACTOR; y++) {
      color src = original.pixels[idxf(x,y)];
      int alf = (int)red(bigMsk.pixels[idxf(x,y)]);
      into.pixels[idxf(x,y)] = color(red(src),green(src),blue(src),alf);
    } 
  }
  into.updatePixels();
}

void captureEvent(Capture cap) {
  redraw();
}

void draw() {

  if(doReset) {
    select(alphaImage);
    doReset = false;
  }
  background(0,0,0);

  captureImage.read();

  for(int x=0; x<W*FACTOR; x++) {
    for(int y=0; y<H*FACTOR; y++) {
      color src = captureImage.pixels[idxf(x,y)];
      flipped.pixels[idxf(W*FACTOR-x,y)] = src;
    } 
  }
  flipped.updatePixels();
  
  sized.copy(flipped,0,0,W*FACTOR,H*FACTOR,0,0,W,H);
  sized.updatePixels();

  analyze(sized,alphaImage);
  reselect(sized,alphaImage);
  alphaImage.filter(BLUR,1);
  alphaImage.filter(THRESHOLD);
  
  ready.copy(alphaImage,0,0,W,H,0,0,W*FACTOR,H*FACTOR);
  ready.filter(BLUR,3);
  resynth(flipped,ready,result);
  
  flipped.filter(BLUR,1);
  image(flipped,0,0);
  fill(0,64);
  rect(0,0,W*FACTOR,H*FACTOR);
  image(result,0,0);
}
